pub struct ApplicationConfigBuilder {
watch_configuration: Option<bool>,
permit_debugging: Option<bool>,
allow_running_as_root: Option<bool>,
}Expand description
A builder to create an instance of ApplicationConfig.
Fields
watch_configuration: Option<bool>In-progress value for watch_configuration.
permit_debugging: Option<bool>In-progress value for permit_debugging.
allow_running_as_root: Option<bool>In-progress value for allow_running_as_root.
Implementations
Sourceimpl ApplicationConfigBuilder
impl ApplicationConfigBuilder
Sourcepub fn watch_configuration(&mut self, val: bool) -> &mut Self
pub fn watch_configuration(&mut self, val: bool) -> &mut Self
Provide a value for watch_configuration.
If no value is provided for watch_configuration, build will use Default::default().
watch_configuration
If true, we should watch our configuration files for changes, and reload our configuration when they change.
Note that this feature may behave in unexpected ways if the path to the directory holding our configuration files changes its identity (because an intermediate symlink is changed, because the directory is removed and recreated, or for some other reason).
Sourcepub fn permit_debugging(&mut self, val: bool) -> &mut Self
pub fn permit_debugging(&mut self, val: bool) -> &mut Self
Provide a value for permit_debugging.
If no value is provided for permit_debugging, build will use Default::default().
permit_debugging
If true, we should allow other applications not owned by the system administrator to monitor the Arti application and inspect its memory.
Otherwise, we take various steps (including disabling core dumps) to make it harder for other programs to view our internal state.
This option has no effect when arti is built without the harden
feature. When harden is not enabled, debugger attachment is permitted
whether this option is set or not.
Sourcepub fn allow_running_as_root(&mut self, val: bool) -> &mut Self
pub fn allow_running_as_root(&mut self, val: bool) -> &mut Self
Sourceimpl ApplicationConfigBuilder
impl ApplicationConfigBuilder
Sourcepub fn build(&self) -> Result<ApplicationConfig, ConfigBuildError>
pub fn build(&self) -> Result<ApplicationConfig, ConfigBuildError>
Try to construct a new ApplicationConfig from the fields set in this builder.
Return an error if any required field is missing, or is set to something invalid.