struct Proxy {
svc: Arc<RunningOnionService>,
proxy: Arc<OnionServiceReverseProxy>,
}Expand description
A running onion service and an associated reverse proxy.
This is what a user configures when they add an onion service to their configuration.
Fields
svc: Arc<RunningOnionService>The onion service.
This is launched and running.
proxy: Arc<OnionServiceReverseProxy>The reverse proxy that accepts connections from the onion service.
This is also launched and running.
Implementations
Sourceimpl Proxy
impl Proxy
Sourcepub(crate) fn launch_new<R: Runtime>(
client: &TorClient<R>,
config: OnionServiceProxyConfig,
) -> Result<Option<Self>>
pub(crate) fn launch_new<R: Runtime>( client: &TorClient<R>, config: OnionServiceProxyConfig, ) -> Result<Option<Self>>
Create and launch a new onion service proxy, using a given client,
to handle connections according to config.
Returns Ok(None) if the service specified is disabled in the config.
Sourcefn reconfigure(
&mut self,
config: OnionServiceProxyConfig,
how: Reconfigure,
) -> Result<(), ReconfigureError>
fn reconfigure( &mut self, config: OnionServiceProxyConfig, how: Reconfigure, ) -> Result<(), ReconfigureError>
Reconfigure this proxy, using the new configuration config and the
rules in how.
Sourcefn reconfigure_inner(
&mut self,
config: OnionServiceProxyConfig,
how: Reconfigure,
) -> Result<(), ReconfigureError>
fn reconfigure_inner( &mut self, config: OnionServiceProxyConfig, how: Reconfigure, ) -> Result<(), ReconfigureError>
Helper for reconfigure: Run reconfigure on each part of this Proxy.