pub struct ListenDefaultsBuilder {
values: Option<Vec<String>>,
}Expand description
Wrapper struct to help derive_builder find the right types and methods
This struct is not part of the configuration API. Refer to the containing structures for information on how to build the config.
Fields
values: Option<Vec<String>>The list, as overridden
Implementations
Sourceimpl ListenDefaultsBuilder
impl ListenDefaultsBuilder
Sourcepub fn build(&self) -> Result<Vec<String>, ConfigBuildError>
pub fn build(&self) -> Result<Vec<String>, ConfigBuildError>
Resolve this list to a list of built items.
If the value is still the Default,
a built-in default list will be built and returned;
otherwise each applicable item will be built,
and the results collected into a single built list.
Sourcefn default_list() -> Vec<String>
fn default_list() -> Vec<String>
The default list
Sourcepub fn access(&mut self) -> &mut Vec<String>
pub fn access(&mut self) -> &mut Vec<String>
Resolve the list to the default if necessary and then return &mut Vec
Sourcepub fn access_opt(&self) -> &Option<Vec<String>>
pub fn access_opt(&self) -> &Option<Vec<String>>
Resolve the list to the default if necessary and then return &mut Vec
Sourcepub fn access_opt_mut(&mut self) -> &mut Option<Vec<String>>
pub fn access_opt_mut(&mut self) -> &mut Option<Vec<String>>
Resolve the list to the default if necessary and then return &mut Vec