Struct PreemptiveCircuitConfigBuilder
pub struct PreemptiveCircuitConfigBuilder {
disable_at_threshold: Option<usize>,
initial_predicted_ports: PreemptiveCircuitConfigInitialPredictedPortsListBuilder,
prediction_lifetime: Option<Duration>,
min_exit_circs_for_port: Option<usize>,
}Expand description
A builder to create an instance of PreemptiveCircuitConfig.
Fields
disable_at_threshold: Option<usize>initial_predicted_ports: PreemptiveCircuitConfigInitialPredictedPortsListBuilderprediction_lifetime: Option<Duration>min_exit_circs_for_port: Option<usize>Implementations
impl PreemptiveCircuitConfigBuilder
impl PreemptiveCircuitConfigBuilder
pub fn disable_at_threshold(
&mut self,
val: usize,
) -> &mut PreemptiveCircuitConfigBuilder
pub fn disable_at_threshold( &mut self, val: usize, ) -> &mut PreemptiveCircuitConfigBuilder
Provide a value for disable_at_threshold.
If no value is provided for disable_at_threshold, build will use Default::default().
disable_at_threshold
If we have at least this many available circuits, we suspend construction of preemptive circuits. whether our available circuits support our predicted exit ports or not.
pub fn prediction_lifetime(
&mut self,
val: Duration,
) -> &mut PreemptiveCircuitConfigBuilder
pub fn prediction_lifetime( &mut self, val: Duration, ) -> &mut PreemptiveCircuitConfigBuilder
Provide a value for prediction_lifetime.
If no value is provided for prediction_lifetime, build will use Default::default().
prediction_lifetime
After we see the client request a connection to a new port, how long should we predict that the client will still want to have circuits available for that port?
pub fn min_exit_circs_for_port(
&mut self,
val: usize,
) -> &mut PreemptiveCircuitConfigBuilder
pub fn min_exit_circs_for_port( &mut self, val: usize, ) -> &mut PreemptiveCircuitConfigBuilder
impl PreemptiveCircuitConfigBuilder
impl PreemptiveCircuitConfigBuilder
pub fn initial_predicted_ports(&mut self) -> &mut Vec<u16>
pub fn initial_predicted_ports(&mut self) -> &mut Vec<u16>
Access the being-built list (resolving default)
If the field has not yet been set or accessed, the default list will be constructed and a mutable reference to the now-defaulted list of builders will be returned.
pub fn set_initial_predicted_ports(&mut self, list: Vec<u16>)
pub fn set_initial_predicted_ports(&mut self, list: Vec<u16>)
Set the whole list (overriding the default)
pub fn opt_initial_predicted_ports(&self) -> &Option<Vec<u16>>
pub fn opt_initial_predicted_ports(&self) -> &Option<Vec<u16>>
Inspect the being-built list (with default unresolved)
If the list has not yet been set, or accessed, &None is returned.
pub fn opt_initial_predicted_ports_mut(&mut self) -> &mut Option<Vec<u16>>
pub fn opt_initial_predicted_ports_mut(&mut self) -> &mut Option<Vec<u16>>
Mutably access the being-built list (with default unresolved)
If the list has not yet been set, or accessed, &mut None is returned.
impl PreemptiveCircuitConfigBuilder
impl PreemptiveCircuitConfigBuilder
pub fn new() -> PreemptiveCircuitConfigBuilder
pub fn new() -> PreemptiveCircuitConfigBuilder
Return a new builder object.
pub fn build(&self) -> Result<PreemptiveCircuitConfig, ConfigBuildError>
pub fn build(&self) -> Result<PreemptiveCircuitConfig, ConfigBuildError>
Try to construct a new PreemptiveCircuitConfig from the fields set in this builder.
Return an error if any required field is missing, or is set to something invalid.