#[non_exhaustive]pub struct BridgesConfigBuilder {
enabled: Option<BoolOrAuto>,
bridges: BridgeListBuilder,
transports: TransportConfigListBuilder,
}Expand description
Builder for BridgesConfig.
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.enabled: Option<BoolOrAuto>Should we use configured bridges?
The default (Auto) is to use bridges if they are configured.
false means to not use even configured bridges.
true means to insist on the use of bridges;
if none are configured, that’s then an error.
bridges: BridgeListBuilderConfigured list of bridges (possibly via pluggable transports)
transports: TransportConfigListBuilderpt-client only.Configured list of pluggable transports.
Implementations
Sourceimpl BridgesConfigBuilder
impl BridgesConfigBuilder
Sourcepub fn enabled(&mut self, value: BoolOrAuto) -> &mut Self
pub fn enabled(&mut self, value: BoolOrAuto) -> &mut Self
Should we use configured bridges?
The default (Auto) is to use bridges if they are configured.
false means to not use even configured bridges.
true means to insist on the use of bridges;
if none are configured, that’s then an error.
Sourcepub fn build(&self) -> Result<BridgesConfig, ConfigBuildError>
pub fn build(&self) -> Result<BridgesConfig, ConfigBuildError>
Sourcefn create_empty() -> Self
fn create_empty() -> Self
Create an empty builder, with all fields set to None or PhantomData.
Sourceimpl BridgesConfigBuilder
impl BridgesConfigBuilder
Sourcepub fn transports(&mut self) -> &mut Vec<TransportConfigBuilder>
pub fn transports(&mut self) -> &mut Vec<TransportConfigBuilder>
Access the being-built list (resolving default)
If the field has not yet been set or accessed, the default list will be constructed and a mutable reference to the now-defaulted list of builders will be returned.
Sourcepub fn set_transports(&mut self, list: Vec<TransportConfigBuilder>)
pub fn set_transports(&mut self, list: Vec<TransportConfigBuilder>)
Set the whole list (overriding the default)
Sourcepub fn opt_transports(&self) -> &Option<Vec<TransportConfigBuilder>>
pub fn opt_transports(&self) -> &Option<Vec<TransportConfigBuilder>>
Inspect the being-built list (with default unresolved)
If the list has not yet been set, or accessed, &None is returned.
Sourcepub fn opt_transports_mut(&mut self) -> &mut Option<Vec<TransportConfigBuilder>>
pub fn opt_transports_mut(&mut self) -> &mut Option<Vec<TransportConfigBuilder>>
Mutably access the being-built list (with default unresolved)
If the list has not yet been set, or accessed, &mut None is returned.
Sourceimpl BridgesConfigBuilder
impl BridgesConfigBuilder
Sourcepub fn bridges(&mut self) -> &mut Vec<BridgeConfigBuilder>
pub fn bridges(&mut self) -> &mut Vec<BridgeConfigBuilder>
Access the being-built list (resolving default)
If the field has not yet been set or accessed, the default list will be constructed and a mutable reference to the now-defaulted list of builders will be returned.
Sourcepub fn set_bridges(&mut self, list: Vec<BridgeConfigBuilder>)
pub fn set_bridges(&mut self, list: Vec<BridgeConfigBuilder>)
Set the whole list (overriding the default)
Sourcepub fn opt_bridges(&self) -> &Option<Vec<BridgeConfigBuilder>>
pub fn opt_bridges(&self) -> &Option<Vec<BridgeConfigBuilder>>
Inspect the being-built list (with default unresolved)
If the list has not yet been set, or accessed, &None is returned.
Sourcepub fn opt_bridges_mut(&mut self) -> &mut Option<Vec<BridgeConfigBuilder>>
pub fn opt_bridges_mut(&mut self) -> &mut Option<Vec<BridgeConfigBuilder>>
Mutably access the being-built list (with default unresolved)
If the list has not yet been set, or accessed, &mut None is returned.