best_hint

Function best_hint 

fn best_hint<'a>(err: &'a (dyn StdError + 'static)) -> Option<ErrorHint<'a>>
Return the best hint possible from err, by looking for the first error in the chain defined by err and its sources that provides a value for HintableErrorImpl::hint.