macro_rules! hintable_impl {
{ $( $e:ty ),+ $(,)? } => { ... };
}Expand description
Declare one or more error types as having hints.
This macro implements Sealed for those types, and makes them participate
in downcast_to_hintable_impl.
macro_rules! hintable_impl {
{ $( $e:ty ),+ $(,)? } => { ... };
}Declare one or more error types as having hints.
This macro implements Sealed for those types, and makes them participate
in downcast_to_hintable_impl.