Struct Bug
pub struct Bug(Box<BugRepr>);Expand description
Programming error (a bug)
Tuple Fields
0: Box<BugRepr>Implementations
impl Bug
impl Bug
pub fn new<S>(kind: ErrorKind, message: S) -> Bug
pub fn new<S>(kind: ErrorKind, message: S) -> Bug
Create a bug error report capturing this call site and backtrace
Prefer to use internal!,
as that makes it easy to add additional information
via format parameters.
pub fn from_error<E, S>(kind: ErrorKind, source: E, message: S) -> Bug
pub fn from_error<E, S>(kind: ErrorKind, source: E, message: S) -> Bug
Create an bug error report from another error, capturing this call site and backtrace
In map_err, and perhaps elsewhere, prefer to use into_internal!,
as that makes it easy to add additional information
via format parameters.
pub fn bug_context(self, prepend: impl Display) -> Bug
pub fn bug_context(self, prepend: impl Display) -> Bug
Adds context to an internal error or bug
Prepends prepend + ": " to the message.