pub struct ArtiNativeKeystore {
keystore_dir: CheckedDir,
id: KeystoreId,
}keymgr only.Expand description
The Arti key store.
This is a disk-based key store that encodes keys in OpenSSH format.
Some of the key types supported by the ArtiNativeKeystore
don’t have a predefined SSH public key algorithm name,
so we define several custom SSH algorithm names.
As per RFC4251  6, our custom SSH algorithm names use the
<something@subdomain.torproject.org> format.
We have assigned the following custom algorithm names:
x25519@spec.torproject.org, for x25519 keysed25519-expanded@spec.torproject.org, for expanded ed25519 keys
See SSH protocol extensions for more details.
Fields
keystore_dir: CheckedDirThe root of the key store.
All the keys are stored within this directory.
id: KeystoreIdThe unique identifier of this instance.
Implementations
Sourceimpl ArtiNativeKeystore
impl ArtiNativeKeystore
Sourcepub fn from_path_and_mistrust(
keystore_dir: impl AsRef<Path>,
mistrust: &Mistrust,
) -> Result<Self>
pub fn from_path_and_mistrust( keystore_dir: impl AsRef<Path>, mistrust: &Mistrust, ) -> Result<Self>
Create a new ArtiNativeKeystore rooted at the specified keystore_dir directory.
The keystore_dir directory is created if it doesn’t exist.
This function returns an error if keystore_dir is not a directory, if it does not conform
to the requirements of the specified Mistrust, or if there was a problem creating the
directory.
Sourcefn rel_path(
&self,
key_spec: &dyn KeySpecifier,
item_type: &KeystoreItemType,
) -> StdResult<RelKeyPath<'_>, ArtiPathUnavailableError>
fn rel_path( &self, key_spec: &dyn KeySpecifier, item_type: &KeystoreItemType, ) -> StdResult<RelKeyPath<'_>, ArtiPathUnavailableError>
The path on disk of the key with the specified identity and type, relative to
keystore_dir.