pub struct RelayIdentities {
pub(crate) rsa_id: RsaIdentity,
pub(crate) ed_id: Ed25519Identity,
pub(crate) cert_id_sign_ed: Ed25519Cert,
pub(crate) cert_sign_tls_ed: Ed25519Cert,
pub(crate) cert_sign_link_auth_ed: Ed25519Cert,
pub(crate) cert_id_x509_rsa: Vec<u8>,
pub(crate) cert_id_rsa: RsaCrosscert,
}relay only.Expand description
Object containing the key and certificate that basically identifies us as a relay. They are used for channel authentication.
We use this intermediary object in order to not have tor-proto crate have access to the KeyMgr meaning access to all keys. This restricts the view to what is needed.
Fields
rsa_id: RsaIdentityAs a relay, our RSA identity key: KP_relayid_rsa
ed_id: Ed25519IdentityAs a relay, our Ed identity key: KP_relayid_ed
cert_id_sign_ed: Ed25519CertThe Ed25519 identity signing cert (CertType 4)
cert_sign_tls_ed: Ed25519CertThe Ed25519 signing TLS cert (CertType 5)
cert_sign_link_auth_ed: Ed25519CertThe Ed25519 signing link auth cert (CertType 6)
cert_id_x509_rsa: Vec<u8>Legacy: the RSA identity X509 cert (CertType 2). We only have the bytes here as create_legacy_rsa_id_cert() takes a key and gives us back the encoded cert.
cert_id_rsa: RsaCrosscertLegacy: the RSA identity cert (CertType 7)
Implementations
Sourceimpl RelayIdentities
impl RelayIdentities
Sourcepub fn new(
rsa_id: RsaIdentity,
ed_id: Ed25519Identity,
cert_id_sign_ed: Ed25519Cert,
cert_sign_tls_ed: Ed25519Cert,
cert_sign_link_auth_ed: Ed25519Cert,
cert_id_x509_rsa: Vec<u8>,
cert_id_rsa: RsaCrosscert,
) -> Self
pub fn new( rsa_id: RsaIdentity, ed_id: Ed25519Identity, cert_id_sign_ed: Ed25519Cert, cert_sign_tls_ed: Ed25519Cert, cert_sign_link_auth_ed: Ed25519Cert, cert_id_x509_rsa: Vec<u8>, cert_id_rsa: RsaCrosscert, ) -> Self
Constructor.
Sourceimpl RelayIdentities
impl RelayIdentities
Sourcepub(crate) fn ed_id_bytes(&self) -> [u8; 32]
pub(crate) fn ed_id_bytes(&self) -> [u8; 32]
Return our Ed identity key (KP_relayid_ed) as bytes.
Sourcepub(crate) fn rsa_x509_digest(&self) -> [u8; 32]
pub(crate) fn rsa_x509_digest(&self) -> [u8; 32]
Return the digest of the RSA x509 certificate (CertType 2) as bytes.