is_zero

Function is_zero 

pub(crate) fn is_zero(x: &[u8]) -> bool
Returns true if all bytes of the input are zero (including if the slice is empty). Executes in constant time for a given length of input.