enum CircuitEvent {
RunCmd {
leg: UniqId,
cmd: CircuitCmd,
},
HandleControl(CtrlMsg),
HandleCell {
leg: UniqId,
cell: ClientCircChanMsg,
},
RemoveLeg {
leg: UniqId,
reason: RemoveLegReason,
},
PaddingAction {
leg: UniqId,
padding_event: PaddingEvent,
},
ProtoViolation {
err: Error,
},
}Expand description
A command to execute at the end of Reactor::run_once.
Variants
RunCmd
Run a single CircuitCmd command.
Fields
cmd: CircuitCmdThe command to run.
HandleControl(CtrlMsg)
Handle a control message
HandleCell
Handle an input message.
Fields
cell: ClientCircChanMsgThe message to handle.
RemoveLeg
Remove the specified circuit leg from the conflux set.
Returned whenever a single circuit leg needs to be be removed from the reactor’s conflux set, without necessarily tearing down the whole set or shutting down the reactor.
Note: this event can cause the reactor to shut down (and the conflux set to be closed).
See the ConfluxSet::remove docs for more on the exact behavior of this command.
Fields
reason: RemoveLegReasonThe reason for removal.
This is only used for conflux circuits that get removed before the conflux handshake is complete.
The RemoveLegReason is mapped by the reactor to a
ConfluxHandshakeError that is sent to the initiator of the
handshake to indicate the reason the handshake failed.
PaddingAction
Take some event (blocking or unblocking a circuit, or sending padding) based on the circuit padding backend code.
Fields
padding_event: PaddingEventThe event to take.
ProtoViolation
Protocol violation. This leads for now to the close of the circuit reactor. The error causing the violation is set in err.
Implementations
Sourceimpl CircuitEvent
impl CircuitEvent
Sourcefn order_within_batch(&self) -> u8
fn order_within_batch(&self) -> u8
Return the ordering with which we should handle this event within a list of events returned by a single call to next_circ_event().
NOTE: Please do not make this any more complicated:
It is a consequence of a kludge that we need this sorting at all.
Assuming that eventually, we switch away from the current
poll-oriented next_circ_event design,
we may be able to get rid of this entirely.