pub(crate) struct SinkBlocker<S, P = BooleanPolicy> {
inner: S,
policy: P,
waker: Option<Waker>,
}Expand description
A wrapper for a [futures::Sink] that allows its blocking status to be
turned on and off according to a policy.
While the policy is blocking, attempts to enqueue data on the sink
via this Sink trait will return Poll::Pending.
Later, when the policy is replaced with a nonblocking one via Self::update_policy()
this sink can be written to again.
Fields
inner: SThe inner sink.
policy: PA policy state object, deciding whether we are blocking or not.
Invariant: Whenever we try to send with a blocking Policy,
we store the context’s waker in self.waker.
If later the policy becomes non-blocking,
we we alert the Waker.
waker: Option<Waker>A waker that we should alert when policy transitions from
a blocking to a non-blocking state.
Implementations
Sourceimpl<S> SinkBlocker<S, BooleanPolicy>
impl<S> SinkBlocker<S, BooleanPolicy>
Sourcepub(crate) fn set_blocked(&mut self)
pub(crate) fn set_blocked(&mut self)
Put this SinkBlocker into a blocked state.
Sourcepub(crate) fn set_unblocked(&mut self)
pub(crate) fn set_unblocked(&mut self)
Put this SinkBlocker into an unblocked state.
Sourceimpl<S> SinkBlocker<S, CountingPolicy>
impl<S> SinkBlocker<S, CountingPolicy>
Sourcepub(crate) fn set_blocked(&mut self)
pub(crate) fn set_blocked(&mut self)
Put this SinkBlocker into a blocked state.
Sourcepub(crate) fn set_unlimited(&mut self)
pub(crate) fn set_unlimited(&mut self)
Put this SinkBlocker into an unlimited state.
Sourcepub(crate) fn allow_n_additional_items(&mut self, n: u32)
pub(crate) fn allow_n_additional_items(&mut self, n: u32)
Allow n additional items to bypass the current blocking of this SinkBlocker.
(This function has no effect if the SinkBlocker is currently unlimited.)
Sourcepub(crate) fn is_unlimited(&self) -> bool
pub(crate) fn is_unlimited(&self) -> bool
Return true if there is no limit on this policy.
Sourceimpl<S, P> SinkBlocker<S, P>
impl<S, P> SinkBlocker<S, P>
Sourceimpl<S, P> SinkBlocker<S, P>
impl<S, P> SinkBlocker<S, P>
Sourcepub(crate) fn new(inner: S, policy: P) -> Self
pub(crate) fn new(inner: S, policy: P) -> Self
Construct a new SinkBlocker wrapping a given sink, with a given
initial blocking policy.
Sourcepub(crate) fn as_inner(&self) -> &S
pub(crate) fn as_inner(&self) -> &S
Return a reference to the inner Sink of this object.
See warnings on as_inner_mut.
Sourcepub(crate) fn as_inner_mut(&mut self) -> &mut S
pub(crate) fn as_inner_mut(&mut self) -> &mut S
Return a mutable reference to the inner Sink of this object.
Note that with this method, it is possible to bypass the blocking features
of SinkBlocker. This is an intentional escape hatch.
Sourceimpl<S, P: Policy> SinkBlocker<S, P>
impl<S, P: Policy> SinkBlocker<S, P>
Sourcepub(crate) fn update_policy(&mut self, new_policy: P)
pub(crate) fn update_policy(&mut self, new_policy: P)
Replace the current Policy state object with new_policy.
This method is used to make a blocked SinkBlocker unblocked,
or vice versa.